package cn.iocoder.yudao.module.ecw.controller.admin.currencyRate;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currencyRate.CurrencyRateDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currencyRate.CurrencyRateLogDO;
import cn.iocoder.yudao.module.ecw.service.currencyRate.CurrencyRateService;
import cn.iocoder.yudao.module.ecw.vo.currencyRate.CurrencyRateCreateReqVO;
import cn.iocoder.yudao.module.ecw.vo.currencyRate.CurrencyRateQueryParamVO;
import cn.iocoder.yudao.module.ecw.vo.currencyRate.CurrencyRateUpdateReqVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 货币汇率")
@RequestMapping("/ecw/currency-rate")
public class CurrencyRateController {
    @Autowired
    private CurrencyRateService service;

    @PostMapping("/create")
    @ApiOperation("创建汇率信息")
    @PreAuthorize("@ss.hasPermission('ecw:currency:rate:create')")
    public CommonResult<Boolean> create(@Valid @RequestBody CurrencyRateCreateReqVO req) {
        service.create(req);
        return success(true);
    }

    @PutMapping("/update")
    @ApiOperation("更新汇率信息")
    @PreAuthorize("@ss.hasPermission('ecw:currency:rate:update')")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> update(@RequestParam("id") Long id, @Valid @RequestBody CurrencyRateUpdateReqVO req) {
        service.update(id, req);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除汇率信息")
    @PreAuthorize("@ss.hasPermission('ecw:currency:rate:delete')")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> delete(@RequestParam("id") Long id) {
        service.delete(id);
        return success(true);
    }

    @GetMapping("/page")
    @ApiOperation("分页查询汇率信息")
    @PreAuthorize("@ss.hasPermission('ecw:currency:rate:query')")
    public CommonResult<PageResult<CurrencyRateDO>> page(@Valid CurrencyRateQueryParamVO param) {
        return success(service.query(param));
    }

    @GetMapping("/get")
    @ApiOperation("根据币种获取汇率信息")
    @PreAuthorize("@ss.hasPermission('ecw:currency:rate:query')")
    public CommonResult<CurrencyRateDO> get(@RequestParam("sourceId") Long sourceId, @RequestParam("targetId") Long targetId) {
        return success(service.find(sourceId, targetId));
    }

    @GetMapping("/logs")
    @ApiOperation("根据币种获取汇率根据货币符号")
    @PreAuthorize("@ss.hasPermission('ecw:currency:rate:history')")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<PageResult<CurrencyRateLogDO>> logs(@RequestParam("id") Long id, PageParam page) {
        return success(service.history(id, page));
    }
}
