package cn.iocoder.yudao.module.ecw.service.api;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.ecw.convert.currency.CurrencyConvert;
import cn.iocoder.yudao.module.ecw.convert.currencyRate.CurrencyRateConvert;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currencyRate.CurrencyRateDO;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.ecw.service.currencyRate.CurrencyRateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class CurrecyApiImpl implements CurrencyApi {

    @Resource
    CurrencyService currencyService;

    @Autowired
    private CurrencyRateService currencyRateService;

    @Override
    public Map<Integer, CurrencyRespDTO> getAllCurrency() {
        return currencyService.getCurrencyList().stream().collect(Collectors.toMap(CurrencyDO::getId, t -> CurrencyConvert.INSTANCE.convert2(t)));
    }

    @Override
    public ExchangeRateRespDTO getCurrencyRate(Long sourceId, Long targetId) {
        ExchangeRateRespDTO dto = new ExchangeRateRespDTO();
        dto.setSourceCurrencyId(sourceId);
        dto.setTargetCurrencyId(targetId);
        if (sourceId == null || targetId == null) {
            dto.setCurrencyRate(BigDecimal.ZERO);
            dto.setSourceCurrencyAmount(BigDecimal.ONE);// 除数默认值给1，避免计算错误
            dto.setTargetCurrencyAmount(BigDecimal.ZERO);
        } else {
            if (sourceId.equals(targetId)){
                dto.setSourceCurrencyAmount(BigDecimal.ONE);
                dto.setTargetCurrencyAmount(BigDecimal.ONE);
                dto.setCurrencyRate(BigDecimal.ONE);
            }else {
                CurrencyRateDO currencyRateDO = currencyRateService.exchangeBaseValue(sourceId, targetId);
                dto.setSourceCurrencyAmount(currencyRateDO.getSourceAmount());
                dto.setTargetCurrencyAmount(currencyRateDO.getTargetAmount());
                dto.setCurrencyRate(currencyRateService.rate(currencyRateDO.getSourceAmount(), currencyRateDO.getTargetAmount()));
            }

        }

        return dto;
    }

    @Override
    public List<ExchangeRateRespDTO> getCurrencyRateList(Collection<Long> sourceIds, Long targetId) {
        return sourceIds.stream()
                .map(sourceId -> getCurrencyRate(sourceId, targetId))
                .collect(Collectors.toList());
    }

    @Override
    public ExchangeRateRespDTO getCurrencyRateByCode(String sourceCode, String targetCode) {
        ExchangeRateRespDTO dto = new ExchangeRateRespDTO();

        dto.setSourceCurrencyCode(sourceCode);
        dto.setTargetCurrencyCode(targetCode);

        if (StrUtil.equals(sourceCode, targetCode)) {
            dto.setSourceCurrencyAmount(BigDecimal.ONE);
            dto.setTargetCurrencyAmount(BigDecimal.ONE);
            dto.setCurrencyRate(BigDecimal.ONE);
            return dto;
        }

        Integer sourceId = Optional.ofNullable(currencyService.getCurrencyByCode(sourceCode))
                .map(CurrencyDO::getId)
                .orElse(null);
        Integer targetId = Optional.ofNullable(currencyService.getCurrencyByCode(targetCode))
                .map(CurrencyDO::getId)
                .orElse(null);

        if (sourceId == null || targetId == null) {
            dto.setCurrencyRate(BigDecimal.ZERO);
            dto.setSourceCurrencyAmount(BigDecimal.ONE);// 除数默认值给1，避免计算错误
            dto.setTargetCurrencyAmount(BigDecimal.ZERO);
        } else {
            if (sourceId.equals(targetId)){
                dto.setSourceCurrencyAmount(BigDecimal.ONE);
                dto.setTargetCurrencyAmount(BigDecimal.ONE);
                dto.setCurrencyRate(BigDecimal.ONE);
            }else {
                CurrencyRateDO currencyRateDO = currencyRateService.exchangeBaseValue(sourceId, targetId);
                dto.setSourceCurrencyAmount(currencyRateDO.getSourceAmount());
                dto.setTargetCurrencyAmount(currencyRateDO.getTargetAmount());
                dto.setCurrencyRate(currencyRateService.rate(currencyRateDO.getSourceAmount(), currencyRateDO.getTargetAmount()));
            }

        }

        return dto;
    }

    @Override
    public List<ExchangeRateRespDTO> getAllCurrencyRate() {
        List<CurrencyRateDO> currencyRateDOS = currencyRateService.queryAll();
        return CurrencyRateConvert.INSTANCE.convertList2(currencyRateDOS);
    }
}
